function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var reactRedux = require('react-redux');
var reselect = require('reselect');
var _ = _interopDefault(require('lodash'));
var redux = require('redux');

var storage = {
  store: null
};
var setStore = function setStore(store) {
  return storage.store = store;
};

var SET_REDUX_STATE = 'SET_REDUX_STATE';
var UNSUBSCRIBE_REDUX_STATE = 'UNSUBSCRIBE_REDUX_STATE';
var SUBSCRIBE_REDUX_STATE = 'SUBSCRIBE_REDUX_STATE';
var CLEANUP_REDUX_STATE = 'CLEANUP_REDUX_STATE';
var STATE_NAME = '_use_redux_state';

var sel = function sel(state) {
  return state;
};

var useMemoSelector = function useMemoSelector(selector, select, eq) {
  if (select === void 0) {
    select = sel;
  }

  if (eq === void 0) {
    eq = _.isEqual;
  }

  return reactRedux.useSelector(reselect.createSelector(selector, select), eq);
};
var useReduxState = function useReduxState(name, initState) {
  var store = react.useRef(storage.store).current;
  var dispatch = reactRedux.useDispatch();
  var action = react.useCallback(function (payload) {
    return {
      type: SET_REDUX_STATE,
      payload: payload,
      name: name
    };
  }, [name]);
  var cleanUpAction = react.useCallback(function (payload) {
    return {
      type: CLEANUP_REDUX_STATE,
      payload: payload,
      name: name
    };
  }, [name]);
  var stateSubscriptionAction = react.useCallback(function (payload) {
    return {
      type: SUBSCRIBE_REDUX_STATE,
      payload: payload,
      name: name
    };
  }, [name]);
  var stateUnSubscriptionAction = react.useCallback(function (payload) {
    return {
      type: UNSUBSCRIBE_REDUX_STATE,
      payload: payload,
      name: name
    };
  }, [name]);
  var setState = react.useCallback(function (payload) {
    return dispatch(action(typeof payload === 'function' ? payload(getState()) : payload));
  }, []);
  var selector = react.useCallback(function (state) {
    var _state$STATE_NAME;

    var storeState = state === null || state === void 0 ? void 0 : (_state$STATE_NAME = state[STATE_NAME]) === null || _state$STATE_NAME === void 0 ? void 0 : _state$STATE_NAME[name];
    return storeState !== undefined ? storeState : initState !== undefined ? initState : '';
  }, [name]);
  var cleanup = react.useCallback(function () {
    return dispatch(cleanUpAction());
  }, [cleanUpAction]);
  var getState = react.useCallback(function () {
    var _store$getState, _store$getState$STATE;

    return store === null || store === void 0 ? void 0 : (_store$getState = store.getState()) === null || _store$getState === void 0 ? void 0 : (_store$getState$STATE = _store$getState[STATE_NAME]) === null || _store$getState$STATE === void 0 ? void 0 : _store$getState$STATE[name];
  }, [name]);
  var getSateSubscription = react.useCallback(function () {
    var _store$getState2, _store$getState2$STAT, _store$getState2$STAT2;

    return (store === null || store === void 0 ? void 0 : (_store$getState2 = store.getState()) === null || _store$getState2 === void 0 ? void 0 : (_store$getState2$STAT = _store$getState2[STATE_NAME]) === null || _store$getState2$STAT === void 0 ? void 0 : (_store$getState2$STAT2 = _store$getState2$STAT.redux_state_subscriptions) === null || _store$getState2$STAT2 === void 0 ? void 0 : _store$getState2$STAT2[name]) || 0;
  }, [name]);
  react.useEffect(function () {
    var subCount = getSateSubscription();
    subCount < 1 && initState !== undefined && setState(initState);
    dispatch(stateSubscriptionAction());
    return function () {
      dispatch(stateUnSubscriptionAction());
    };
  }, []);
  return {
    selector: selector,
    setState: setState,
    getState: getState,
    action: action,
    cleanup: cleanup
  };
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var INIT_STATE = {};

var merge = function merge(obj, obj2) {
  return Object.assign({}, obj, obj2);
};

var reducer = function reducer(state, _ref) {
  var _merge, _state, _state2, _state2$redux_state_s, _state4, _state4$redux_state_s, _state5, _merge3, _state6;

  if (state === void 0) {
    state = INIT_STATE;
  }

  var type = _ref.type,
      payload = _ref.payload,
      name = _ref.name;

  switch (type) {
    case SET_REDUX_STATE:
      return merge(state, (_merge = {}, _merge[name] = payload, _merge));

    case UNSUBSCRIBE_REDUX_STATE:
      var redux_state_subscriptions = (_state = state) === null || _state === void 0 ? void 0 : _state.redux_state_subscriptions;
      var subscriber_count = ((_state2 = state) === null || _state2 === void 0 ? void 0 : (_state2$redux_state_s = _state2.redux_state_subscriptions) === null || _state2$redux_state_s === void 0 ? void 0 : _state2$redux_state_s[name]) || 0;

      if (subscriber_count < 2) {
        if (state && state[name]) {
          delete state[name];
        }

        if (redux_state_subscriptions && redux_state_subscriptions[name]) {
          delete redux_state_subscriptions[name];
        }

        return _extends({}, state);
      } else {
        var _state3, _merge2;

        return merge(state, {
          redux_state_subscriptions: merge(((_state3 = state) === null || _state3 === void 0 ? void 0 : _state3.redux_state_subscriptions) || {}, (_merge2 = {}, _merge2[name] = subscriber_count - 1, _merge2))
        });
      }

    case SUBSCRIBE_REDUX_STATE:
      var subscribed_count = ((_state4 = state) === null || _state4 === void 0 ? void 0 : (_state4$redux_state_s = _state4.redux_state_subscriptions) === null || _state4$redux_state_s === void 0 ? void 0 : _state4$redux_state_s[name]) || 0;
      return merge(state, {
        redux_state_subscriptions: merge(((_state5 = state) === null || _state5 === void 0 ? void 0 : _state5.redux_state_subscriptions) || {}, (_merge3 = {}, _merge3[name] = subscribed_count + 1, _merge3))
      });

    case CLEANUP_REDUX_STATE:
      if (state && state[name]) {
        delete state[name];
      }

      if (((_state6 = state) === null || _state6 === void 0 ? void 0 : _state6.redux_state_subscriptions) && state.redux_state_subscriptions[name]) {
        var _state$redux_state_su;

        (_state$redux_state_su = state.redux_state_subscriptions) === null || _state$redux_state_su === void 0 ? true : delete _state$redux_state_su[name];
      }

      return _extends({}, state);

    default:
      return state;
  }
};

var reducers = (function (baseReducer) {
  var _extends2;

  return redux.combineReducers(_extends({}, baseReducer, (_extends2 = {}, _extends2[STATE_NAME] = reducer, _extends2)));
});

exports.default = useReduxState;
exports.mergeReducers = reducers;
exports.setStore = setStore;
exports.useMemoSelector = useMemoSelector;
exports.useReduxState = useReduxState;
//# sourceMappingURL=index.js.map
